/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2019
* 
*        (as is).
*     .
********************************************************************************
*/



#include "usbd_audio_hid.h"


USBD_Class_cb_TypeDef  USBD_AUDIO_HID_cb = 
{
  USBD_AUDIO_HID_Init,
  USBD_AUDIO_HID_DeInit,
  USBD_AUDIO_HID_Setup,
  NULL,  
  USBD_AUDIO_HID_EP0_RxReady, 
  USBD_AUDIO_HID_DataIn, 
  USBD_AUDIO_HID_DataOut,
  USBD_AUDIO_HID_SOF, 
  USBD_AUDIO_HID_GetConfigDescriptor, 
};




/* HID Device library callbacks */
extern uint8_t  USBD_HID_Init (void  *pdev,  uint8_t cfgidx);
extern uint8_t  USBD_HID_DeInit (void  *pdev, uint8_t cfgidx);
extern uint8_t  USBD_HID_Setup (void  *pdev, USB_SETUP_REQ *req);
extern uint8_t  USBD_HID_DataIn (void  *pdev, uint8_t epnum);
extern uint8_t  USBD_HID_DataOut (void  *pdev, uint8_t epnum);
extern uint8_t  USBD_HID_EP0_RxReady(void *pdev);


/* USB AUDIO_HID device Configuration Descriptor */
const uint8_t USBD_AUDIO_HID_CfgDesc[USB_AUDIO_HID_CONFIG_DESC_SIZ] =
{
  0x09, /* bLength: Configuration Descriptor size */
  USB_CONFIGURATION_DESCRIPTOR_TYPE, /* bDescriptorType: Configuration */
  USB_AUDIO_HID_CONFIG_DESC_SIZ,
  /* wTotalLength: Bytes returned */
  0x00,
  0x03,         /*bNumInterfaces: 4 interfaces (3 Audio, 1 for HID)*/
  0x01,         /*bConfigurationValue: Configuration value*/
  0x00,         /*iConfiguration: Index of string descriptor describing
  the configuration*/
  0xC0,         /*bmAttributes: bus powered and Support Remote Wake-up */
  0x32,         /*MaxPower 100 mA: this current is used for detecting Vbus*/
  
  //IAD
  0x08, /* bLength */
  0x0B, /* bDescriptorType */
  0x00, /* bFirstInterface */
  0x02, /* bInterfaceCount */
  USB_DEVICE_CLASS_AUDIO, /* bFunctionClass */
  AUDIO_SUBCLASS_AUDIOCONTROL, /* bFunctionSubClass */
  AUDIO_PROTOCOL_UNDEFINED, /* bFunctionProtocol */
  0x00, /* iFunction (Index of string descriptor describing this function) */
  
  
  /* USB Speaker Standard interface descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOCONTROL,          /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Class-specific AC Interface Descriptor */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_HEADER,                 /* bDescriptorSubtype */
  0x00,          /* 1.00 */             /* bcdADC */
  0x01,
  0x27,                                 /* wTotalLength = 39*/
  0x00,
  0x01,                                 /* bInCollection */
  0x01,                                 /* baInterfaceNr */
  /* 09 byte*/

  /* USB Speaker Input Terminal Descriptor */
  AUDIO_INPUT_TERMINAL_DESC_SIZE,       /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_INPUT_TERMINAL,         /* bDescriptorSubtype */
  0x01,                                 /* bTerminalID */
  0x01,                                 /* wTerminalType AUDIO_TERMINAL_USB_STREAMING   0x0101 */
  0x01,
  0x00,                                 /* bAssocTerminal */
  0x02,                                 /* bNrChannels */
  0x03,                                 /* wChannelConfig 0x0000  Mono */
  0x00,
  0x00,                                 /* iChannelNames */
  0x00,                                 /* iTerminal */
  /* 12 byte*/

  /* USB Speaker Audio Feature Unit Descriptor */
  0x09,                                 /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_FEATURE_UNIT,           /* bDescriptorSubtype */
  AUDIO_OUT_STREAMING_CTRL,             /* bUnitID */
  0x01,                                 /* bSourceID */
  0x01,                                 /* bControlSize */
  AUDIO_CONTROL_MUTE,                   /* bmaControls(0) */
  0x00,                                 /* bmaControls(1) */
  0x00,                                 /* iTerminal */
  /* 09 byte*/

  /*USB Speaker Output Terminal Descriptor */
  0x09,      /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_CONTROL_OUTPUT_TERMINAL,        /* bDescriptorSubtype */
  0x03,                                 /* bTerminalID */
  0x01,                                 /* wTerminalType  0x0301*/
  0x03,
  0x00,                                 /* bAssocTerminal */
  0x02,                                 /* bSourceID */
  0x00,                                 /* iTerminal */
  /* 09 byte*/

  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Zero Bandwidth */
  /* Interface 1, Alternate Setting 0                                             */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
  /* 09 byte*/

  
  /* USB Speaker Standard AS Interface Descriptor - Audio Streaming Operational */
  /* Interface 1, Alternate Setting 1                                           */
  AUDIO_INTERFACE_DESC_SIZE,            /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x01,                                 /* bInterfaceNumber */
  0x01,                                 /* bAlternateSetting */
  0x01,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_AUDIO,               /* bInterfaceClass */
  AUDIO_SUBCLASS_AUDIOSTREAMING,        /* bInterfaceSubClass */
  AUDIO_PROTOCOL_UNDEFINED,             /* bInterfaceProtocol */
  0x01,                                 /* iInterface */
  /* 09 byte*/

  /* USB Speaker Audio Streaming Interface Descriptor */
  AUDIO_STREAMING_INTERFACE_DESC_SIZE,  /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_GENERAL,              /* bDescriptorSubtype */
  0x01,                                 /* bTerminalLink */
  0x01,                                 /* bDelay */
  0x01,                                 /* wFormatTag AUDIO_FORMAT_PCM  0x0001 */
  0x00,
  /* 07 byte*/

  /* USB Speaker Audio Type I Format Interface Descriptor */
  (0x08 + (3 * SUPPORTED_FREQ_24BIT_NBR)),    /* bLength */
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,      /* bDescriptorType */
  AUDIO_STREAMING_FORMAT_TYPE,          /* bDescriptorSubtype */
  AUDIO_FORMAT_TYPE_I,                  /* bFormatType */
  DEFAULT_OUT_CHANNEL_NBR,              /* bNrChannels */
  3,                                    /* bSubFrameSize :  3 Bytes per frame (24bits) */
  24,                                   /* bBitResolution (24-bits per sample) */
  SUPPORTED_FREQ_24BIT_NBR,                    /* bSamFreqType only one frequency supported */
  SAMPLE_FREQ(DEFAULT_OUT_AUDIO_FREQ),   /* Audio sampling frequency coded on 3 bytes */
 /* 11 + 3*N byte */
  //20 max
  
  /* USB Speaker Streaming Standard Endpoint Descriptor */
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,    /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,         /* bDescriptorType */
  AUDIO_OUT_EP,                         /* bEndpointAddress 1 out endpoint*/
  USB_ENDPOINT_TYPE_ISOCHRONOUS,        /* bmAttributes */
  LOBYTE(AUDIO_OUT_MPS),                /* wMaxPacketSize */
  HIBYTE(AUDIO_OUT_MPS),
  0x01,                                 /* bInterval */
  0x00,                                 /* bRefresh */
  0x00,                                 /* bSynchAddress */
  /* 09 byte*/

  /* Endpoint - Audio Streaming Descriptor*/
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,   /* bLength */
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,       /* bDescriptorType */
  AUDIO_ENDPOINT_GENERAL,               /* bDescriptor */
#if (SUPPORTED_FREQ_24BIT_NBR > 1)  
  0x01,                                 /* bmAttributes: Freq selection */
#else
  0x00,                                 /* bmAttributes: None */
#endif /* (SUPPORTED_FREQ_NBR > 1)  */
  0x00,                                 /* bLockDelayUnits */
  0x00,                                 /* wLockDelay */
  0x00,
  /* 07 byte*/  
  
  
  /************** Descriptor of Custom HID interface ****************/
  /* 09 */
  0x09,         /*bLength: Interface Descriptor size*/
  USB_INTERFACE_DESCRIPTOR_TYPE,/*bDescriptorType: Interface descriptor type*/
  0x02,         /*bInterfaceNumber: Number of Interface*/
  0x00,         /*bAlternateSetting: Alternate setting*/
  0x01, //0x02,         /*bNumEndpoints*/
  0x03,         /*bInterfaceClass: HID*/
  0x00,         /*bInterfaceSubClass : 1=BOOT, 0=no boot*/
  0x00,         /*nInterfaceProtocol : 0=none, 1=keyboard, 2=mouse*/
  1,            /*iInterface: Index of string descriptor*/
  /******************** Descriptor of Custom HID ********************/
  /* 18 */
  0x09,         /*bLength: HID Descriptor size*/
  HID_DESCRIPTOR_TYPE, /*bDescriptorType: HID*/
  0x11,         /*bcdHID: HID Class Spec release number*/
  0x01,
  0x00,         /*bCountryCode: Hardware target country*/
  0x01,         /*bNumDescriptors: Number of HID class descriptors to follow*/
  0x22,         /*bDescriptorType*/
  CUSTOMHID_SIZ_REPORT_DESC,/*wItemLength: Total length of Report descriptor*/
  0x00,
  
  /******************** Descriptor of Custom HID endpoints ***********/
  /* 27 */
  0x07,          /* bLength: Endpoint Descriptor size */
  USB_ENDPOINT_DESCRIPTOR_TYPE, /* bDescriptorType: */
  
  HID_IN_EP,     /* bEndpointAddress: Endpoint Address (IN) */
  0x03,          /* bmAttributes: Interrupt endpoint */
  HID_IN_PACKET, /* wMaxPacketSize: 2 Bytes max */
  0x00,
  0x64,          /* bInterval: Polling Interval*/
  /* 34 */
  
};


uint8_t USBD_AUDIO_HID_Init(void  *pdev, uint8_t cfgidx)
{
  usbd_audio_Init(pdev,cfgidx);
  
  USBD_HID_Init(pdev,cfgidx);
  
  return USBD_OK;

}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_DeInit(void  *pdev, uint8_t cfgidx)
{
  usbd_audio_DeInit(pdev,cfgidx);
  
  USBD_HID_DeInit(pdev,cfgidx);
    
  return USBD_OK;
}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_Setup(void  *pdev, USB_SETUP_REQ *req)
{
  switch (req->bmRequest & USB_REQ_RECIPIENT_MASK)
  {
    case USB_REQ_RECIPIENT_INTERFACE:
    {
      if (req->wIndex == HID_INTERFACE)
        return (USBD_HID_Setup (pdev, req));
      else
        return (usbd_audio_Setup(pdev, req));
    }
    case USB_REQ_RECIPIENT_ENDPOINT:
    {
      if (req->wIndex == HID_IN_EP)
        return (USBD_HID_Setup (pdev, req));
      else
        return (usbd_audio_Setup(pdev, req));
    }
  }
  
  return USBD_OK;
}
//------------------------------------------------------------------------------
uint8_t *USBD_AUDIO_HID_GetConfigDescriptor(uint8_t speed, uint16_t *length)
{
  *length = sizeof(USBD_AUDIO_HID_CfgDesc);
  return (uint8_t*)USBD_AUDIO_HID_CfgDesc;
}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_DataIn(void  *pdev, uint8_t epnum)
{  
  if (epnum == (HID_IN_EP&~0x80) )
    return (USBD_HID_DataIn(pdev, epnum));
  else
    return (usbd_audio_DataIn(pdev, epnum));
}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_DataOut(void *pdev , uint8_t epnum)
{
  if (epnum == (AUDIO_OUT_EP&~0x80) )
    return (usbd_audio_DataOut(pdev, epnum));
  else
    return (USBD_HID_DataOut(pdev, epnum));
}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_SOF(void *pdev)
{
  return (usbd_audio_SOF(pdev));
}
//------------------------------------------------------------------------------
uint8_t USBD_AUDIO_HID_EP0_RxReady(void *pdev)
{
  (usbd_audio_EP0_RxReady(pdev));
  (USBD_HID_EP0_RxReady(pdev));
  return USBD_OK;
}




